'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UPDATE_PACKAGE_ACCESS_TOKENS = exports.UPDATE_ACCESS_TOKEN = undefined;
exports.prepareRequest = prepareRequest;
exports.default = request;
exports.download = download;
exports.isPackageAvailable = isPackageAvailable;
exports.getUrlForPackage = getUrlForPackage;
exports.makeRefreshAccessTokenRequest = makeRefreshAccessTokenRequest;
exports.refreshAccessToken = refreshAccessToken;
exports.parseResponse = parseResponse;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _immutable = require('immutable');

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _fileSaver = require('file-saver');

var _fileSaver2 = _interopRequireDefault(_fileSaver);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _imsSharedCore = require('ims-shared-core');

var _cookies = require('./cookies');

var _headers = require('./headers');

var HeaderUtils = _interopRequireWildcard(_headers);

var _RequestError = require('./RequestError');

var _RequestError2 = _interopRequireDefault(_RequestError);

var _window = require('./window');

var _auth = require('./auth');

var _PackageInformation = require('../PackageInformation');

var _PackageInformation2 = _interopRequireDefault(_PackageInformation);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var REFRESH_ATTEMPT_DELAY = 5000;
var refreshAttempts = 0;

var storeInstance = void 0;
var packagesPathKey = void 0;
var tokensPathKey = void 0;
var hubUrl = void 0;

var REFRESH_ACCESS_TOKEN_TIMEOUT = void 0;

var UPDATE_ACCESS_TOKEN = exports.UPDATE_ACCESS_TOKEN = 'UPDATE_ACCESS_TOKEN';
var UPDATE_PACKAGE_ACCESS_TOKENS = exports.UPDATE_PACKAGE_ACCESS_TOKENS = 'UPDATE_PACKAGE_ACCESS_TOKENS';

function prepareRequest(store, tokensPath, packagesPath) {
  if (store && store.getState && store.dispatch) {
    storeInstance = store;
  } else {
    throw new TypeError('Invalid "store" should be an instance of a redux store');
  }
  if (typeof packagesPath === 'string') {
    packagesPathKey = packagesPath.split('.');
  } else if (Array.isArray(packagesPath)) {
    packagesPathKey = packagesPath;
  } else {
    throw new TypeError('Invalid "packagesPath" should be either a string or an array');
  }
  if (typeof tokensPath === 'string') {
    tokensPathKey = tokensPath.split('.');
  } else if (Array.isArray(tokensPath)) {
    tokensPathKey = tokensPath;
  } else {
    throw new TypeError('Invalid "tokensPath" should be either a string or an array');
  }

  hubUrl = (0, _cookies.getCookie)(_imsSharedCore.Constants.Cookies.HubUrl);
  if (!hubUrl) {
    throw new Error('"HUB_URL" cookie has not been set');
  }

  refreshAttempts = 0;

  return refreshAccessToken();
}

function request(url, options) {
  try {
    verifyInitialized();

    var opts = prepareOptions(options);

    if (opts.authenticate) {
      return makeAuthenticatedRequest(url, opts.requestOptions);
    } else {
      return makeUnauthenticatedRequest(url, opts.requestOptions);
    }
  } catch (e) {
    return _bluebird2.default.reject(e);
  }
}

function download(url, options) {
  try {
    verifyInitialized();

    var opts = prepareDownloadOptions(options);

    if (opts.authenticate) {
      return makeAuthenticatedDownload(url, opts.requestOptions, opts.defaultFileName);
    } else {
      return makeUnauthentiatedDownload(url, opts.requestOptions, opts.defaultFileName);
    }
  } catch (e) {
    return _bluebird2.default.reject(e);
  }
}

function getTokens() {
  var tokens = storeInstance.getState().getIn(tokensPathKey);
  return _immutable.Map.isMap(tokens) ? tokens.toJS() : undefined;
}

function getAccessToken() {
  var tokens = getTokens();
  if (tokens) {
    if (tokens.expires < (0, _moment2.default)().add(1, 'minute').unix()) {
      return refreshAccessToken().then(function (response) {
        return response.accessToken;
      });
    } else {
      return _bluebird2.default.resolve(tokens.accessToken);
    }
  } else {
    console.log('NO TOKENS');
    throw new Error('No tokens in the store');
  }
}

function isPackageAvailable(packageId) {
  var packagesList = storeInstance.getState().getIn(packagesPathKey);

  var packages = _immutable.List.isList(packagesList) ? packagesList.toJS() : undefined;
  if (packages) {
    var filtered = packages.filter(function (p) {
      return p.packageId === packageId;
    });
    return filtered.length > 0 && filtered[0].installed;
  }
  return false;
}

function getPackage(packageId) {
  var packagesList = storeInstance.getState().getIn(packagesPathKey);

  var packages = _immutable.List.isList(packagesList) ? packagesList.toJS() : undefined;
  if (packages) {
    var filtered = packages.filter(function (p) {
      return p.packageId === packageId;
    });
    if (filtered.length > 0) {
      return filtered[0];
    }
  }
  return undefined;
}

function getAccessTokenForPackage(packageId) {
  var pkg = getPackage(packageId);

  if (pkg) {
    var token = pkg.accessToken;

    if (token) {
      if (token.expires < (0, _moment2.default)().add(1, 'minute').unix()) {
        return refreshAccessToken().then(function () {
          return getAccessTokenForPackage(packageId);
        });
      } else {
        return _bluebird2.default.resolve(token.token);
      }
    }
  } else {
    console.log('NO PACKAGE');
    throw new Error('The requested package is not present in the store.  You need to add ' + packageId + ' to your packageDependencies.');
  }
}

function scheduleRefreshAccessToken() {
  verifyInitialized();

  var tokens = getTokens();

  if (tokens) {
    var refreshTime = _moment2.default.unix(tokens.expires).subtract(5, 'minutes');
    var refreshIn = refreshTime.diff((0, _moment2.default)());

    if (refreshIn < 0) {
      refreshIn = 0;
    }
    if (REFRESH_ACCESS_TOKEN_TIMEOUT) {
      clearTimeout(REFRESH_ACCESS_TOKEN_TIMEOUT);
    }
    REFRESH_ACCESS_TOKEN_TIMEOUT = setTimeout(function () {
      return refreshAccessToken(true);
    }, refreshIn);
  } else {
    throw new Error('No tokens in the store');
  }
}

function verifyInitialized() {
  if (!storeInstance || !tokensPathKey || !_PackageInformation2.default.packageId || !hubUrl) {
    throw new Error('Request has not been prepared. You need to call "prepareRequest" to configure the request.');
  }
}

function combineUrlParts(base, endpoint) {
  if (endpoint.indexOf('/') !== 0) {
    endpoint = '/' + endpoint;
  }
  if (base.lastIndexOf('/') === base.length - 1) {
    base = base.substr(0, base.length - 1);
  }
  return '' + base + endpoint;
}

function getUrlForPackage(packageId, endpoint) {
  var pkg = getPackage(packageId);
  if (pkg && pkg.packageUrl) {
    return combineUrlParts(pkg.packageUrl, endpoint);
  } else {
    return undefined;
  }
}

function makeAuthenticatedRequest(url, requestOptions) {
  if (requestOptions.packageId) {
    var pkg = getPackage(requestOptions.packageId);
    if (pkg && pkg.packageUrl) {
      return getAccessTokenForPackage(pkg.packageId).then(function (accessToken) {
        requestOptions.headers = HeaderUtils.createAuthenticatedRequestHeader(requestOptions.packageId, accessToken);

        return makeRequest(combineUrlParts(pkg.packageUrl, url), requestOptions);
      });
    } else {
      throw new Error('Package ' + requestOptions.packageId + ' was not found in ' + packagesPathKey.join('.') + '.  Make sure you have added the package to your packageDependencies');
    }
  } else {
    return getAccessToken().then(function (accessToken) {
      requestOptions.headers = HeaderUtils.createAuthenticatedRequestHeader(_PackageInformation2.default.packageId, accessToken);
      return makeRequest(url, requestOptions);
    });
  }
}

function makeUnauthenticatedRequest(url, requestOptions) {
  requestOptions.headers = HeaderUtils.createRequestHeader(_PackageInformation2.default.packageId);
  return makeRequest(url, requestOptions);
}

function makeRequest(url, requestOptions) {
  return (0, _isomorphicFetch2.default)(url, requestOptions).then(parseResponse);
}

function makeAuthenticatedDownload(url, requestOptions, defaultFileName) {
  return getAccessToken().then(function (accessToken) {
    requestOptions.headers = HeaderUtils.createAuthenticatedRequestHeader(_PackageInformation2.default.packageId, accessToken);
    return makeDownloadRequest(url, requestOptions, defaultFileName);
  });
}

function makeUnauthentiatedDownload(url, requestOptions, defaultFileName) {
  requestOptions.headers = HeaderUtils.createRequestHeader(_PackageInformation2.default.packageId);
  return makeDownloadRequest(url, requestOptions, defaultFileName);
}

function makeDownloadRequest(url, requestOptions, defaultFileName) {
  return (0, _isomorphicFetch2.default)(url, requestOptions).then(parseDownloadResponse(defaultFileName));
}

function makeRefreshAccessTokenRequest() {
  var refreshToken = (0, _cookies.getCookie)(_imsSharedCore.Constants.Cookies.RefreshToken);

  if (!refreshToken) {
    return _bluebird2.default.resolve({ success: false, message: 'Refresh token not found.' });
  }

  var body = { refreshToken: refreshToken };
  var accessTokenEndpoint = 'accessToken';

  // If a package has packageDependencies set, use those
  if (_PackageInformation2.default.packageDependencies) {
    var packages = Array.isArray(_PackageInformation2.default.packageDependencies) ? _PackageInformation2.default.packageDependencies : Object.keys(_PackageInformation2.default.packageDependencies);
    if (packages.indexOf(_PackageInformation2.default.packageId) === -1) {
      packages.push(_PackageInformation2.default.packageId);
    }
    body.packages = packages;
    accessTokenEndpoint = 'accessTokens';
  }
  var refreshOptions = {
    headers: HeaderUtils.createRequestHeader(_PackageInformation2.default.packageId),
    method: 'POST',
    body: JSON.stringify(body)
  };

  return (0, _isomorphicFetch2.default)(window.__HUB_URL__ + '/users/' + accessTokenEndpoint, refreshOptions).then(parseResponse);
}

function parseDownloadResponse(defaultFileName) {
  return function (response) {
    if (response.status === 200) {
      var contentType = response.headers.get('content-type');

      if (contentType.indexOf('application/json') === -1) {
        return response.blob().then(function (blob) {
          var contentDisposition = response.headers.get('content-disposition');
          var re = /filename[^;=\n]*=((['"]).*?\2|[^;\n]*)/;
          var result = re.exec(contentDisposition);
          if (result && result[1]) {
            defaultFileName = result[1].replace(/"/g, '').replace(/'/g, '');
          }
          _fileSaver2.default.saveAs(blob, defaultFileName);
          return true;
        });
      }
    }

    return response.json();
  };
}

function refreshAccessToken(scheduleRefresh) {
  return makeRefreshAccessTokenRequest().then(function (response) {
    if (response.success === true) {
      if (response.results) {
        for (var i = 0; i < response.results.length; i++) {
          if (response.results[i].packageId === _PackageInformation2.default.packageId) {
            if (!response.results[i].accessToken || !response.results[i].accessToken.token) {
              return goToLogin(response.results[i].loginUrl);
            }
            storeInstance.dispatch(updateAccessToken(response.results[i].accessToken.token, response.results[i].accessToken.expires));
          }
        }
        storeInstance.dispatch(updatePackageAccessTokens(response.results));
      } else {
        if (!response.accessToken) {
          return goToLogin(response.loginUrl);
        }
        storeInstance.dispatch(updateAccessToken(response.accessToken, response.expires));
      }

      scheduleRefreshAccessToken();

      refreshAttempts = 0;
    } else {
      console.error('Received success false from refreshAccessToken with message: ' + response.message);
      return goToLogin(response.loginUrl);
    }

    return response;
  }).catch(function (error) {
    refreshAttempts++;
    console.error('Failed refreshing access token attempt ' + refreshAttempts, error);
    setTimeout(function () {
      return refreshAccessToken(true);
    }, REFRESH_ATTEMPT_DELAY * refreshAttempts);
  });
}

function parseResponse(response) {
  if (!response || response.status >= 500) {
    throw new _RequestError2.default('Response received a server error');
  }
  var contentType = response.headers.get('content-type');
  if (contentType && contentType.indexOf('application/json') !== -1) {
    return response.json();
  } else {
    return response.status === 200;
  }
}

function updatePackageAccessTokens(accessTokens) {
  return {
    type: UPDATE_PACKAGE_ACCESS_TOKENS,
    accessTokens: accessTokens
  };
}

function updateAccessToken(accessToken, expires) {
  return {
    type: UPDATE_ACCESS_TOKEN,
    accessToken: accessToken,
    expires: expires
  };
}

function prepareDownloadOptions() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var opts = prepareOptions(options);

  if (!opts.defaultFileName) {
    opts.defaultFileName = 'download';
  }

  return opts;
}

function prepareOptions() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var opts = Object.assign({}, options);

  if (!opts.method) {
    opts.method = 'GET';
  }

  if (!opts.credentials) {
    opts.credentials = 'same-origin';
  }

  if (opts.authenticate === undefined) {
    opts.authenticate = true;
  }

  var requestOptions = {
    method: opts.method,
    credentials: opts.credentials,
    packageId: opts.packageId,
    body: opts.body,
    headers: {
      'content-type': 'application/json',
      accept: 'application/json'
    }
  };

  if (!opts.requestOptions) {
    opts.requestOptions = requestOptions;
  } else {
    opts.requestOptions = Object.assign({}, requestOptions, opts.requestOptions);
  }

  return opts;
}

function goToLogin(loginUrl) {
  loginUrl = loginUrl || (0, _auth.getAuthorizeUrl)(window.location.pathname);
  (0, _window.setLocation)(loginUrl);
  return { success: false, message: 'Invalid refreshToken redirecting to login' };
}